@@ -1,8 +1,8 @@
-# $Id$
+# $Id: PrimarySeq.pm,v 1.95.4.4 2006/10/02 23:10:12 sendu Exp $
 #
 # bioperl module for Bio::PrimarySeq
 #
-# Cared for by Ewan Birney <birney@sanger.ac.uk>
+# Cared for by Ewan Birney <birney@ebi.ac.uk>
 #
 # Copyright Ewan Birney
 #
@@ -16,62 +16,62 @@ Bio::PrimarySeq - Bioperl lightweight Sequence Object
 
 =head1 SYNOPSIS
 
-  # The Bio::SeqIO for file reading, Bio::DB::GenBank for
+  # Bio::SeqIO for file reading, Bio::DB::GenBank for
   # database reading
 
   use Bio::Seq;
   use Bio::SeqIO;
   use Bio::DB::GenBank;
 
-  #make from memory
+  # make from memory
+
   $seqobj = Bio::PrimarySeq->new ( -seq => 'ATGGGGTGGGCGGTGGGTGGTTTG',
-				   -id  => 'GeneFragment-12',
-				   -accession_number => 'X78121',
-				   -alphabet => 'dna',
-				   -is_circular => 1
-				   );
-  print "Sequence ", $seqobj->id(), " with accession ", 
+                                   -id  => 'GeneFragment-12',
+                                   -accession_number => 'X78121',
+                                   -alphabet => 'dna',
+                                   -is_circular => 1 );
+  print "Sequence ", $seqobj->id(), " with accession ",
     $seqobj->accession_number, "\n";
 
   # read from file
-  $inputstream = Bio::SeqIO->new(-file => "myseq.fa",-format => 'Fasta');
+
+  $inputstream = Bio::SeqIO->new(-file => "myseq.fa",
+                                 -format => 'Fasta');
   $seqobj = $inputstream->next_seq();
   print "Sequence ", $seqobj->id(), " and desc ", $seqobj->desc, "\n";
 
-
   # to get out parts of the sequence.
 
-  print "Sequence ", $seqobj->id(), " with accession ", 
+  print "Sequence ", $seqobj->id(), " with accession ",
     $seqobj->accession_number, " and desc ", $seqobj->desc, "\n";
 
   $string  = $seqobj->seq();
   $string2 = $seqobj->subseq(1,40);
 
-
 =head1 DESCRIPTION
 
-PrimarySeq is a lightweight Sequence object, storing little more than
-the sequence, its name, a computer useful unique name. It does not
-contain sequence features or other information.  To have a sequence
-with sequence features you should use the Seq object which uses this
-object - go perldoc Bio::Seq
+PrimarySeq is a lightweight Sequence object, storing the sequence, its
+name, a computer-useful unique name, and other fundamental attributes.
+It does not contain sequence features or other information.  To have a
+sequence with sequence features you should use the Seq object which uses
+this object.
 
-Although newusers will use Bio::PrimarySeq alot, in general you will
+Although new users will use Bio::PrimarySeq a lot, in general you will
 be using it from the Bio::Seq object. For more information on Bio::Seq
-go perldoc Bio::Seq. For interest you might like to known that
+see L<Bio::Seq>. For interest you might like to know that
 Bio::Seq has-a Bio::PrimarySeq and forwards most of the function calls
 to do with sequence to it (the has-a relationship lets us get out of a
 otherwise nasty cyclical reference in Perl which would leak memory).
 
 Sequence objects are defined by the Bio::PrimarySeqI interface, and this
-object is a pure Perl implementation of the interface (if that's
+object is a pure Perl implementation of the interface. If that's
 gibberish to you, don't worry. The take home message is that this
 object is the bioperl default sequence object, but other people can
-use their own objects as sequences if they so wish). If you are
+use their own objects as sequences if they so wish. If you are
 interested in wrapping your own objects as compliant Bioperl sequence
 objects, then you should read the Bio::PrimarySeqI documentation
 
-The documenation of this object is a merge of the Bio::PrimarySeq and
+The documentation of this object is a merge of the Bio::PrimarySeq and
 Bio::PrimarySeqI documentation.  This allows all the methods which you can
 call on sequence objects here.
 
@@ -83,23 +83,20 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to one
 of the Bioperl mailing lists.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org             - General discussion
-  http://bio.perl.org/MailList.html - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-the bugs and their resolution.  Bug reports can be submitted via email
-or the web:
+the bugs and their resolution.  Bug reports can be submitted via the
+web:
 
-  bioperl-bugs@bio.perl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Ewan Birney
 
-Email birney@sanger.ac.uk
-
-Describe contact details here
+Email birney@ebi.ac.uk
 
 =head1 APPENDIX
 
@@ -113,16 +110,12 @@ methods. Internal methods are usually preceded with a _
 
 
 package Bio::PrimarySeq;
-use vars qw(@ISA);
+use vars qw($MATCHPATTERN);
 use strict;
 
-use Bio::Root::Root;
-use Bio::PrimarySeqI;
-use Bio::IdentifiableI;
-use Bio::DescribableI;
+$MATCHPATTERN = 'A-Za-z\-\.\*\?=~';
 
-@ISA = qw(Bio::Root::Root Bio::PrimarySeqI
-	  Bio::IdentifiableI Bio::DescribableI);
+use base qw(Bio::Root::Root Bio::PrimarySeqI Bio::IdentifiableI Bio::DescribableI);
 
 #
 # setup the allowed values for alphabet()
@@ -148,12 +141,13 @@ my %valid_type = map {$_, 1} qw( dna rna protein );
            values.
  Returns : a new Bio::PrimarySeq object
  Args    : -seq         => sequence string
-           -display_id  => display id of the sequence (locus name) 
+           -display_id  => display id of the sequence (locus name)
            -accession_number => accession number
            -primary_id  => primary id (Genbank id)
            -namespace   => the namespace for the accession
            -authority   => the authority for the namespace
-           -desc        => description text
+           -description => description text
+           -desc        => alias for description
            -alphabet    => sequence type (alphabet) (dna|rna|protein)
            -id          => alias for display id
            -is_circular => boolean field for whether or not sequence is circular
@@ -166,7 +160,8 @@ sub new {
     my $self = $class->SUPER::new(@args);
 
     my($seq,$id,$acc,$pid,$ns,$auth,$v,$oid,
-       $desc,$alphabet,$given_id,$is_circular,$direct,$ref_to_seq,$len) =
+       $desc,$description,
+       $alphabet,$given_id,$is_circular,$direct,$ref_to_seq,$len) =
 	$self->_rearrange([qw(SEQ
 			      DISPLAY_ID
 			      ACCESSION_NUMBER
@@ -176,6 +171,7 @@ sub new {
 			      VERSION
 			      OBJECT_ID
 			      DESC
+			      DESCRIPTION
 			      ALPHABET
 			      ID
 			      IS_CIRCULAR
@@ -185,10 +181,10 @@ sub new {
 			      )],
 			  @args);
     if( defined $id && defined $given_id ) {
-	if( $id ne $given_id ) {
-	    $self->throw("Provided both id and display_id constructor ".
-			 "functions. [$id] [$given_id]");	
-	}
+		 if( $id ne $given_id ) {
+			 $self->throw("Provided both id and display_id constructor ".
+							  "functions. [$id] [$given_id]");
+		 }
     }
     if( defined $given_id ) { $id = $given_id; }
 
@@ -199,25 +195,26 @@ sub new {
     # if alphabet is provided we set it first, so that it won't be guessed
     # when the sequence is set
     $alphabet && $self->alphabet($alphabet);
-    
+
     # if there is an alphabet, and direct is passed in, assumme the alphabet
-    # and sequence is ok 
+    # and sequence is ok
 
     if( $direct && $ref_to_seq) {
-	$self->{'seq'} = $$ref_to_seq;
-	if( ! $alphabet ) {
-	    $self->_guess_alphabet();
-	} # else it has been set already above
+		 $self->{'seq'} = $$ref_to_seq;
+		 if( ! $alphabet ) {
+		     $self->_guess_alphabet();
+		 } # else it has been set already above
     } else {
-#	print STDERR "DEBUG: setting sequence to [$seq]\n";
-	# note: the sequence string may be empty
-	$self->seq($seq) if defined($seq);
-    }
+		 #	print STDERR "DEBUG: setting sequence to [$seq]\n";
+		 # note: the sequence string may be empty
+		 $self->seq($seq) if defined($seq);
+	 }
 
     $id          && $self->display_id($id);
     $acc         && $self->accession_number($acc);
     defined $pid && $self->primary_id($pid);
     $desc        && $self->desc($desc);
+    $description && $self->description($description);
     $is_circular && $self->is_circular($is_circular);
     $ns          && $self->namespace($ns);
     $auth        && $self->authority($auth);
@@ -230,7 +227,7 @@ sub new {
 sub direct_seq_set {
     my $obj = shift;
     return $obj->{'seq'} = shift if @_;
-    return undef;
+    return;
 }
 
 
@@ -241,12 +238,10 @@ sub direct_seq_set {
  Function: Returns the sequence as a string of letters. The
            case of the letters is left up to the implementer.
            Suggested cases are upper case for proteins and lower case for
-           DNA sequence (IUPAC standard), but you should not rely on this
+           DNA sequence (IUPAC standard), but you should not rely on this.
  Returns : A scalar
  Args    : Optionally on set the new value (a string). An optional second
            argument presets the alphabet (otherwise it will be guessed).
-           Both parameters may also be given in named paramater style
-           with -seq and -alphabet being the names.
 
 =cut
 
@@ -259,32 +254,31 @@ sub seq {
 
    my ($value,$alphabet) = @args;
 
-
    if(@args) {
        if(defined($value) && (! $obj->validate_seq($value))) {
 	   $obj->throw("Attempting to set the sequence to [$value] ".
-		       "which does not look healthy");
-       }
+							"which does not look healthy");
+		}
        # if a sequence was already set we make sure that we re-adjust the
-       # mol.type, otherwise we skip guessing if mol.type is already set
+       # alphabet, otherwise we skip guessing if alphabet is already set
        # note: if the new seq is empty or undef, we don't consider that a
        # change (we wouldn't have anything to guess on anyway)
-       my $is_changed_seq =
-	   exists($obj->{'seq'}) && (CORE::length($value || '') > 0);
-       $obj->{'seq'} = $value;
+		my $is_changed_seq =
+		  exists($obj->{'seq'}) && (CORE::length($value || '') > 0);
+		$obj->{'seq'} = $value;
        # new alphabet overridden by arguments?
-       if($alphabet) {
+		if($alphabet) {
 	   # yes, set it no matter what
-	   $obj->alphabet($alphabet);
-       } elsif( # if we changed a previous sequence to a new one
-		$is_changed_seq ||
-		# or if there is no alphabet yet at all
-		(! defined($obj->alphabet()))) {
-	   # we need to guess the (possibly new) alphabet
-	   $obj->_guess_alphabet();
-       } # else (seq not changed and alphabet was defined) do nothing
-       # if the seq is changed, make sure we unset a possibly set length
-       $obj->length(undef) if $is_changed_seq;
+			$obj->alphabet($alphabet);
+		} elsif( # if we changed a previous sequence to a new one
+				  $is_changed_seq ||
+				  # or if there is no alphabet yet at all
+				  (! defined($obj->alphabet()))) {
+			# we need to guess the (possibly new) alphabet
+			$obj->_guess_alphabet();
+		} # else (seq not changed and alphabet was defined) do nothing
+		# if the seq is changed, make sure we unset a possibly set length
+		$obj->length(undef) if $is_changed_seq || $obj->{'seq'};
    }
    return $obj->{'seq'};
 }
@@ -293,15 +287,16 @@ sub seq {
 
  Title   : validate_seq
  Usage   : if(! $seq->validate_seq($seq_str) ) {
-                print "sequence $seq_str is not valid for an object of type ",
-		      ref($seq), "\n";
+                print "sequence $seq_str is not valid for an object of
+                alphabet ",$seq->alphabet, "\n";
 	   }
  Function: Validates a given sequence string. A validating sequence string
            must be accepted by seq(). A string that does not validate will
            lead to an exception if passed to seq().
 
            The implementation provided here does not take alphabet() into
-           account. Allowed are all letters (A-Z) and '-','.', '*' and '?'.
+           account. Allowed are all letters (A-Z) and '-','.','*','?','=',
+           and '~'.
 
  Example :
  Returns : 1 if the supplied sequence string is valid for the object, and
@@ -312,15 +307,16 @@ sub seq {
 =cut
 
 sub validate_seq {
-    my ($self,$seqstr) = @_;
-    if( ! defined $seqstr ){ $seqstr = $self->seq(); }
-    return 0 unless( defined $seqstr); 
-    if((CORE::length($seqstr) > 0) && ($seqstr !~ /^([A-Za-z\-\.\*\?]+)$/)) {
-	$self->warn("seq doesn't validate, mismatch is " .
-		   ($seqstr =~ /([^A-Za-z\-\.\*\?]+)/g));
-	return 0;
-    }
-    return 1;
+	my ($self,$seqstr) = @_;
+	if( ! defined $seqstr ){ $seqstr = $self->seq(); }
+	return 0 unless( defined $seqstr);
+	if((CORE::length($seqstr) > 0) &&
+	   ($seqstr !~ /^([$MATCHPATTERN]+)$/)) {
+	    $self->warn("seq doesn't validate, mismatch is " .
+			join(",",($seqstr =~ /([^$MATCHPATTERN]+)/g)));
+		return 0;
+	}
+	return 1;
 }
 
 =head2 subseq
@@ -356,13 +352,14 @@ sub subseq {
        return $seq;
    } elsif(  defined  $start && defined $end ) {
        if( $start > $end ){
-	   $self->throw("in subseq, start [$start] has to be ".
-			"greater than end [$end]");
+	   $self->throw("Bad start,end parameters. Start [$start] has to be ".
+			"less than end [$end]");
        }
-       if( $start <= 0 || $end > $self->length ) {
-	   $self->throw("You have to have start positive\n\tand length less ".
-			"than the total length of sequence [$start:$end] ".
-			"Total ".$self->length."");
+       if( $start <= 0 ) {
+	   $self->throw("Bad start parameter ($start). Start must be positive.");
+       }
+       if( $end > $self->length ) {
+	   $self->throw("Bad end parameter ($end). End must be less than the total length of sequence (total=".$self->length.")");
        }
 
        # remove one from start, and then length is end-start
@@ -373,8 +370,8 @@ sub subseq {
 	   return substr( $self->seq(), $start, ($end-$start));
        }
    } else {
-       $self->warn("Incorrect parameters to subseq - must be two integers ".
-		   "or a Bio::LocationI object not ($start,$end)");
+       $self->warn("Incorrect parameters to subseq - must be two integers or a Bio::LocationI object");
+       return;
    }
 }
 
@@ -408,16 +405,16 @@ sub subseq {
 sub length {
     my $self = shift;
     my $len = CORE::length($self->seq() || '');
-    
+
     if(@_) {
-	my $val = shift;
-	if(defined($val) && $len && ($len != $val)) {
-	    $self->throw("You're trying to lie about the length: ".
-			 "is $len but you say ".$val);
-	}
-	$self->{'_seq_length'} = $val;
+		 my $val = shift;
+		 if(defined($val) && $len && ($len != $val)) {
+			 $self->throw("You're trying to lie about the length: ".
+							  "is $len but you say ".$val);
+		 }
+		 $self->{'_seq_length'} = $val;
     } elsif(defined($self->{'_seq_length'})) {
-	return $self->{'_seq_length'};
+		 return $self->{'_seq_length'};
     }
     return $len;
 }
@@ -452,9 +449,8 @@ sub display_id {
    my ($obj,$value) = @_;
    if( defined $value) {
       $obj->{'display_id'} = $value;
-    }
-    return $obj->{'display_id'};
-
+	}
+	return $obj->{'display_id'};
 }
 
 =head2 accession_number
@@ -473,7 +469,7 @@ sub display_id {
 
            [Note this method name is likely to change in 1.3]
 
-           With the new Bio::IdentifiableI interface, this is aliased 
+           With the new Bio::IdentifiableI interface, this is aliased
            to object_id
 
  Returns : A string
@@ -485,10 +481,10 @@ sub accession_number {
     my( $obj, $acc ) = @_;
 
     if (defined $acc) {
-        $obj->{'accession_number'} = $acc;
+		 $obj->{'accession_number'} = $acc;
     } else {
-        $acc = $obj->{'accession_number'};
-        $acc = 'unknown' unless defined $acc;
+		 $acc = $obj->{'accession_number'};
+		 $acc = 'unknown' unless defined $acc;
     }
     return $acc;
 }
@@ -511,15 +507,15 @@ sub accession_number {
 =cut
 
 sub primary_id {
-   my ($obj,$value) = @_;
-   if( defined $value) {
-      $obj->{'primary_id'} = $value;
-    }
-   if( ! exists $obj->{'primary_id'} ) {
-       return "$obj";
-   }
-   return $obj->{'primary_id'};
+    my $obj = shift;
 
+    if(@_) {
+		 $obj->{'primary_id'} = shift;
+    }
+    if( ! defined($obj->{'primary_id'}) ) {
+		 return "$obj";
+    }
+    return $obj->{'primary_id'};
 }
 
 
@@ -527,16 +523,16 @@ sub primary_id {
 
  Title   : alphabet
  Usage   : if( $obj->alphabet eq 'dna' ) { /Do Something/ }
- Function: Returns the type of sequence being one of
+ Function: Get/Set the alphabet of sequence, one of
            'dna', 'rna' or 'protein'. This is case sensitive.
 
            This is not called <type> because this would cause
            upgrade problems from the 0.5 and earlier Seq objects.
 
  Returns : a string either 'dna','rna','protein'. NB - the object must
-           make a call of the type - if there is no type specified it
+           make a call of the type - if there is no alphabet specified it
            has to guess.
- Args    : none
+ Args    : optional string to set : 'dna' | 'rna' | 'protein'
 
 
 =cut
@@ -544,13 +540,13 @@ sub primary_id {
 sub alphabet {
     my ($obj,$value) = @_;
     if (defined $value) {
-	$value = lc $value;
-	unless ( $valid_type{$value} ) {
-	    $obj->throw("Molecular type '$value' is not a valid type (".
-			join(',', map "'$_'", sort keys %valid_type) .
-			") lowercase");
-	}
-	$obj->{'alphabet'} = $value;
+		 $value = lc $value;
+		 unless ( $valid_type{$value} ) {
+			 $obj->throw("Alphabet '$value' is not a valid alphabet (".
+							 join(',', map "'$_'", sort keys %valid_type) .
+							 ") lowercase");
+		 }
+		 $obj->{'alphabet'} = $value;
     }
     return $obj->{'alphabet'};
 }
@@ -561,7 +557,7 @@ sub alphabet {
  Usage   : $obj->desc($newval)
  Function: Get/set description of the sequence.
 
-           description is an alias for this for compliance with the
+           'description' is an alias for this for compliance with the
            Bio::DescribeableI interface.
 
  Example :
@@ -594,7 +590,6 @@ sub can_call_new {
    my ($self) = @_;
 
    return 1;
-
 }
 
 =head2 id
@@ -625,6 +620,7 @@ sub  id {
 
 sub is_circular{
     my $self = shift;
+
     return $self->{'is_circular'} = shift if @_;
     return $self->{'is_circular'};
 }
@@ -637,9 +633,9 @@ sub is_circular{
 
  Title   : object_id
  Usage   : $string    = $obj->object_id()
- Function: a string which represents the stable primary identifier
+ Function: A string which represents the stable primary identifier
            in this namespace of this object. For DNA sequences this
-           is its accession_number, similarly for protein sequences
+           is its accession_number, similarly for protein sequences.
 
            This is aliased to accession_number().
  Returns : A scalar
@@ -655,10 +651,10 @@ sub object_id {
 
  Title   : version
  Usage   : $version    = $obj->version()
- Function: a number which differentiates between versions of
+ Function: A number which differentiates between versions of
            the same object. Higher numbers are considered to be
            later and more relevant, but a single object described
-           the same identifier should represent the same concept
+           the same identifier should represent the same concept.
 
  Returns : A number
 
@@ -667,7 +663,7 @@ sub object_id {
 sub version{
     my ($self,$value) = @_;
     if( defined $value) {
-	$self->{'_version'} = $value;
+		 $self->{'_version'} = $value;
     }
     return $self->{'_version'};
 }
@@ -677,9 +673,9 @@ sub version{
 
  Title   : authority
  Usage   : $authority    = $obj->authority()
- Function: a string which represents the organisation which
-           granted the namespace, written as the DNS name for  
-           organisation (eg, wormbase.org)
+ Function: A string which represents the organisation which
+           granted the namespace, written as the DNS name for
+           organisation (eg, wormbase.org).
 
  Returns : A scalar
 
@@ -688,7 +684,7 @@ sub version{
 sub authority {
     my ($obj,$value) = @_;
     if( defined $value) {
-	$obj->{'authority'} = $value;
+		 $obj->{'authority'} = $value;
     }
     return $obj->{'authority'};
 }
@@ -699,7 +695,7 @@ sub authority {
  Usage   : $string    = $obj->namespace()
  Function: A string representing the name space this identifier
            is valid in, often the database name or the name
-           describing the collection 
+           describing the collection.
 
  Returns : A scalar
 
@@ -709,7 +705,7 @@ sub authority {
 sub namespace{
     my ($self,$value) = @_;
     if( defined $value) {
-	$self->{'namespace'} = $value;
+		 $self->{'namespace'} = $value;
     }
     return $self->{'namespace'} || "";
 }
@@ -724,11 +720,11 @@ This comprises of display_name and description.
 
  Title   : display_name
  Usage   : $string    = $obj->display_name()
- Function: A string which is what should be displayed to the user
-           the string should have no spaces (ideally, though a cautious
+ Function: A string which is what should be displayed to the user.
+           The string should have no spaces (ideally, though a cautious
            user of this interface would not assumme this) and should be
-           less than thirty characters (though again, double checking 
-           this is a good idea)
+           less than thirty characters (though again, double checking
+           this is a good idea).
 
            This is aliased to display_id().
  Returns : A scalar
@@ -743,12 +739,12 @@ sub display_name {
 
  Title   : description
  Usage   : $string    = $obj->description()
- Function: A text string suitable for displaying to the user a 
+ Function: A text string suitable for displaying to the user a
            description. This string is likely to have spaces, but
            should not have any newlines or formatting - just plain
            text. The string should not be greater than 255 characters
            and clients can feel justified at truncating strings at 255
-           characters for the purposes of display
+           characters for the purposes of display.
 
            This is aliased to desc().
  Returns : A scalar
@@ -771,12 +767,12 @@ actually implemented on Bio::PrimarySeqI
  Function: Produces a new Bio::SeqI implementing object which
            is the reversed complement of the sequence. For protein
            sequences this throws an exception of
-           "Sequence is a protein. Cannot revcom"
+           "Sequence is a protein. Cannot revcom".
 
            The id is the same id as the orginal sequence, and the
            accession number is also indentical. If someone wants to
            track that this sequence has be reversed, it needs to
-           define its own extensions
+           define its own extensions.
 
            To do an inplace edit of an object you can go:
 
@@ -798,7 +794,7 @@ actually implemented on Bio::PrimarySeqI
  Function: Provides a truncation of a sequence,
 
  Example :
- Returns : a fresh Bio::SeqI implementing object
+ Returns : A fresh Bio::SeqI implementing object.
  Args    :
 
 
@@ -814,30 +810,37 @@ These are internal methods to PrimarySeq
 
  Title   : _guess_alphabet
  Usage   :
- Function:
+ Function: Determines (and sets) the type of sequence: dna, rna, protein
  Example :
- Returns :
- Args    :
+ Returns : one of strings 'dna', 'rna' or 'protein'.
+ Args    : none
 
 
 =cut
 
 sub _guess_alphabet {
    my ($self) = @_;
-   my ($str,$str2,$total,$atgc,$u,$type);
+   my $type;
 
-   $str = $self->seq();
-   $str =~ s/\-\.\?//g;
+	#return if $self->alphabet;
 
-   $total = CORE::length($str);
+   my $str = $self->seq();
+	# Remove char's that clearly denote ambiguity
+   $str =~ s/[-.?x]//gi;
+
+   my $total = CORE::length($str);
    if( $total == 0 ) {
-       $self->throw("Got a sequence with no letters in - ".
-		    "cannot guess alphabet [$str]");
+       $self->warn("Got a sequence with no letters in it ".
+		   "cannot guess alphabet [$str]");
+       return '';
    }
-   
-   $u = ($str =~ tr/Uu//);
-   $atgc = ($str =~ tr/ATGCNatgcn//);
-   
+
+   my $u = ($str =~ tr/Uu//);
+	# The assumption here is that most of sequences comprised of mainly
+   # ATGC, with some N, will be 'dna' despite the fact that N could
+	# also be Asparagine
+   my $atgc = ($str =~ tr/ATGCNatgcn//);
+
    if( ($atgc / $total) > 0.85 ) {
        $type = 'dna';
    } elsif( (($atgc + $u) / $total) > 0.85 ) {
