@@ -1,4 +1,4 @@
-# $Id$
+# $Id: FTHelper.pm,v 1.61.4.1 2006/10/02 23:10:28 sendu Exp $
 #
 # BioPerl module for Bio::SeqIO::FTHelper
 #
@@ -16,7 +16,8 @@ Bio::SeqIO::FTHelper - Helper class for Embl/Genbank feature tables
 
 =head1 SYNOPSIS
 
-Used by Bio::SeqIO::EMBL to help process the Feature Table
+Used by Bio::SeqIO::EMBL,Bio::SeqIO::genbank, and Bio::SeqIO::swiss to
+help process the Feature Table
 
 =head1 DESCRIPTION
 
@@ -34,24 +35,20 @@ User feedback is an integral part of the evolution of this and other
 Bioperl modules. Send your comments and suggestions preferably to one
 of the Bioperl mailing lists.  Your participation is much appreciated.
 
-  bioperl-l@bioperl.org                 - General discussion
-  http://www.bioperl.org/MailList.shtml - About the mailing lists
+  bioperl-l@bioperl.org                  - General discussion
+  http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
 
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
-the bugs and their resolution.  Bug reports can be submitted via email
-or the web:
+the bugs and their resolution.  Bug reports can be submitted via the web:
 
-  bioperl-bugs@bio.perl.org
-  http://bugzilla.bioperl.org/
+  http://bugzilla.open-bio.org/
 
 =head1 AUTHOR - Ewan Birney
 
 Email birney@ebi.ac.uk
 
-Describe contact details here
-
 =head1 CONTRIBUTORS
 
 Jason Stajich jason@bioperl.org
@@ -68,7 +65,6 @@ methods. Internal methods are usually preceded with a _
 
 
 package Bio::SeqIO::FTHelper;
-use vars qw(@ISA);
 use strict;
 
 use Bio::SeqFeature::Generic;
@@ -77,18 +73,17 @@ use Bio::Location::Fuzzy;
 use Bio::Location::Split;
 
 
-use Bio::Root::Root;
 
-@ISA = qw(Bio::Root::Root);
+use base qw(Bio::Root::Root);
 
 sub new {
     my ($class, @args) = @_;
 
-    # no chained new because we make lots and lots of these. 
+    # no chained new because we make lots and lots of these.
     my $self = {};
     bless $self,$class;
     $self->{'_field'} = {};
-    return $self; 
+    return $self;
 }
 
 =head2 _generic_seqfeature
@@ -119,26 +114,31 @@ sub _generic_seqfeature {
 
     # parse location; this may cause an exception, in which case we gently
     # recover and ignore this feature
+
+
     my $loc;
     eval {
 	$loc = $locfac->from_string($fth->loc);
     };
+
     if(! $loc) {
 	  $fth->warn("exception while parsing location line [" . $fth->loc .
 		      "] in reading $source, ignoring feature " .
 		      $fth->key() . " (seqid=" . $seqid . "): " . $@);
 	  return;
     }
-    
+
     # set additional location attributes
     if($seqid && (! $loc->is_remote())) {
 	$loc->seq_id($seqid); # propagates if it is a split location
     }
 
+
     # set attributes of feature
     $sf->location($loc);
     $sf->primary_tag($fth->key);
     $sf->source_tag($source);
+    $sf->seq_id($seqid);
     foreach my $key ( keys %{$fth->field} ){
 	foreach my $value ( @{$fth->field->{$key}} ) {
 	    $sf->add_tag_value($key,$value);
@@ -165,68 +165,67 @@ sub _generic_seqfeature {
 =cut
 
 sub from_SeqFeature {
-    my ($sf, $context_annseq) = @_;
-    my @ret;
+  my ($sf, $context_annseq) = @_;
+  my @ret;
 
-    #
-    # If this object knows how to make FThelpers, then let it
-    # - this allows us to store *really* weird objects that can write
-    # themselves to the EMBL/GenBank...
-    #
+  #
+  # If this object knows how to make FThelpers, then let it
+  # - this allows us to store *really* weird objects that can write
+  # themselves to the EMBL/GenBank...
+  #
 
-    if ( $sf->can("to_FTHelper") ) {
+  if ( $sf->can("to_FTHelper") ) {
 	return $sf->to_FTHelper($context_annseq);
-    }
+  }
 
-    my $fth = Bio::SeqIO::FTHelper->new();
-    my $key = $sf->primary_tag();
-    my $locstr = $sf->location->to_FTstring;
-    
-    # ES 25/06/01 Commented out this code, Jason to double check
-    #The location FT string for all simple subseqfeatures is already 
-    #in the Split location FT string
+  my $fth = Bio::SeqIO::FTHelper->new();
+  my $key = $sf->primary_tag();
+  my $locstr = $sf->location->to_FTstring;
 
-    # going into sub features
-    #foreach my $sub ( $sf->sub_SeqFeature() ) {
-	#my @subfth = &Bio::SeqIO::FTHelper::from_SeqFeature($sub);
-	#push(@ret, @subfth);    
-    #}
+  # ES 25/06/01 Commented out this code, Jason to double check
+  #The location FT string for all simple subseqfeatures is already
+  #in the Split location FT string
 
-    $fth->loc($locstr);
-    $fth->key($key);
-    $fth->field->{'note'} = [];
-    #$sf->source_tag && do { push(@{$fth->field->{'note'}},"source=" . $sf->source_tag ); };
-    
-    ($sf->can('score') && $sf->score) && do { push(@{$fth->field->{'note'}},
-						   "score=" . $sf->score ); };
-    ($sf->can('frame') && $sf->frame) && do { push(@{$fth->field->{'note'}},
-						   "frame=" . $sf->frame ); };
-    #$sf->strand && do { push(@{$fth->field->{'note'}},"strand=" . $sf->strand ); };
+  # going into sub features
+  #foreach my $sub ( $sf->sub_SeqFeature() ) {
+  #my @subfth = &Bio::SeqIO::FTHelper::from_SeqFeature($sub);
+  #push(@ret, @subfth);
+  #}
 
-    foreach my $tag ( $sf->all_tags ) {
-        # Tags which begin with underscores are considered
-        # private, and are therefore not printed
-        next if $tag =~ /^_/;
+  $fth->loc($locstr);
+  $fth->key($key);
+  $fth->field->{'note'} = [];
+  #$sf->source_tag && do { push(@{$fth->field->{'note'}},"source=" . $sf->source_tag ); };
+
+  ($sf->can('score') && $sf->score) && do { push(@{$fth->field->{'note'}},
+                                                 "score=" . $sf->score ); };
+  ($sf->can('frame') && $sf->frame) && do { push(@{$fth->field->{'note'}},
+                                                 "frame=" . $sf->frame ); };
+  #$sf->strand && do { push(@{$fth->field->{'note'}},"strand=" . $sf->strand ); };
+
+  foreach my $tag ( $sf->get_all_tags ) {
+    # Tags which begin with underscores are considered
+    # private, and are therefore not printed
+    next if $tag =~ /^_/;
 	if ( !defined $fth->field->{$tag} ) {
-	    $fth->field->{$tag} = [];
+      $fth->field->{$tag} = [];
 	}
-	foreach my $val ( $sf->each_tag_value($tag) ) {
-	    push(@{$fth->field->{$tag}},$val);
+	foreach my $val ( $sf->get_Annotations($tag) ) {
+      push(@{$fth->field->{$tag}},$val);
 	}
-    }
-    push(@ret, $fth);
+  }
+  push(@ret, $fth);
 
-    unless (@ret) {
+  unless (@ret) {
 	$context_annseq->throw("Problem in processing seqfeature $sf - no fthelpers. Error!");
-    }
-    foreach my $ft (@ret) {
+  }
+  foreach my $ft (@ret) {
 	if ( !$ft->isa('Bio::SeqIO::FTHelper') ) {
-	    $sf->throw("Problem in processing seqfeature $sf - made a $fth!");
+      $sf->throw("Problem in processing seqfeature $sf - made a $fth!");
 	}
-    }
-
-    return @ret;
+  }
 
+  return @ret;
 }
 
 
